import sys
import os
from pathlib import Path

try:
    # for package import
    from .config import Config
    from .log_config import log
    from .gui import Gui
    from .api import Api, ApiException
    from .crypt import Crypt
except ModuleNotFoundError:
    # for direct call of clipster.py
    from config import Config
    from log_config import log
    from gui import Gui
    from api import Api, ApiException
    from crypt import Crypt


def get_cred(mygui):
    """ Get credentials, check them and save to config
    """
    log.debug("Asking for credentials.")
    mygui.ask_for_cred()


def set_clip(mygui, api):
    """ Upload current clip to server and display tray notification
    """
    try:
        clip = api.upload()
    except ApiException as e:
        mygui.tray.show_message(
            f"{Config.APP_NAME} - Share Clip Error",
            f"Error sharing Clip:\n{e}",
            data_base64=Config.ICON_B64,
            time=Config.SHOW_MESSAGE_DURATION,
        )
    else:
        mygui.tray.show_message(
            f"{Config.APP_NAME} - shared Clip",
            f"{clip[0:Config.MAX_NOTIFY_LEN]}",
            data_base64=Config.ICON_B64,
            time=Config.SHOW_MESSAGE_DURATION,
        )


def get_clip(mygui, api, all_clips=False):
    """ Download clips from server and display single tray notification
        or list of all Clips
    """
    try:
        clips = api.download()
    except ApiException as e:
        mygui.tray.show_message(
            f"{Config.APP_NAME} - Get Clip Error",
            f"Error downloading Clip:\n{e}",
            data_base64=Config.ICON_B64,
            time=Config.SHOW_MESSAGE_DURATION,
        )
    else:
        if all_clips:
            mygui.show_clip_list_window(clips)
            return True
        else:
            mygui.tray.show_message(
                f"{Config.APP_NAME} - Got Clip",
                f"{clips[-1][0:Config.MAX_NOTIFY_LEN]}",
                data_base64=Config.ICON_B64,
                time=Config.SHOW_MESSAGE_DURATION,
            )
            return True


def deal_with_tray_event(mygui, api, event):
    """ React to menu actions in tray
    """
    log.info(event)
    if event == "Get last Clip":
        get_clip(mygui, api, False)
    if event == "Get all Clips":
        get_clip(mygui, api, True)
    elif event == "Share Clip":
        set_clip(mygui, api)
    elif event == "Edit Credentials":
        get_cred(mygui)
    elif event == "Exit":
        sys.exit(0)


def wait_for_tray_event(mygui, server, username, hash_login, hash_msg):
    """ Check for config modification while waiting for action in systray
    """
    log.debug("Main Loop\n")
    log.debug(f"{server} - {username} - {hash_login} - {hash_msg}")
    api = Api(server, username, hash_login, hash_msg)

    while True:
        if Config.was_configfile_modified():
            log.info("Configfile was modified.")
            self_restart()
        event = mygui.tray.read()
        deal_with_tray_event(mygui, api, event)


def self_restart():
    """ self restart after configuration change
    """
    log.debug("Restarting program")
    python = sys.executable
    os.execl(python, python, *sys.argv)


def main():
    """ Make sure config is valid and start main loop
    """
    mygui = Gui()
    while not Config.is_configfile_valid():
        log.debug("No valid config file")
        get_cred(mygui)
    wait_for_tray_event(
        mygui, Config.SERVER, Config.USER, Config.PW_HASH_LOGIN, Config.PW_HASH_MSG,
    )


if __name__ == "__main__":
    main()



import PySimpleGUIQt as sg
import re

try:
    # for package import
    from .config import Config
    from .log_config import log
    from .crypt import Crypt
    from .api import Api
    from .api import RegisterException, LoginException
except ModuleNotFoundError:
    # for direct call of clipster.py
    from config import Config
    from log_config import log
    from crypt import Crypt
    from api import Api
    from api import RegisterException, LoginException


class Gui:
    # Define Systray menu
    menu_def = [
        "BLANK",
        [
            "&Get last Clip",
            "Get &all Clips",
            "&Share Clip",
            "---",
            "&Edit Credentials",
            "---",
            "E&xit",
        ],
    ]

    # Define own color theme
    sg.LOOK_AND_FEEL_TABLE["Clipster"] = {
        "BACKGROUND": "1234567890",
        "TEXT": "1234567890",
        "INPUT": "1234567890",
        "TEXT_INPUT": "1234567890",
        "SCROLL": "1234567890",
        "BUTTON": ("white", "black"),
        "PROGRESS": ("#01826B", "#D0D0D0"),
        "BORDER": 1,
        "SLIDER_DEPTH": 1,
        "PROGRESS_DEPTH": 0,
    }

    tray = None
    def_server = Config.DEFAULT_SERVER_URI
    def_user = ""
    def_pw = ""

    def __init__(self):
        sg.theme("Clipster")
        self.tray = sg.SystemTray(menu=self.menu_def, data_base64=Config.ICON_B64)

    def create_cred_layout(self):
        """ Layouts in simplepyguy cannot be reused
            need to create new one when (re)-opening windows
            if valid, use configured credentials as default values
        """

        if Config.SERVER and Config.USER and Config.PW:
            self.def_server = Config.SERVER
            self.def_user = Config.USER
            self.def_pw = Config.PW

        layout = [
            [
                sg.Text("Server address: ", size=(30, 1)),
                sg.InputText(default_text=self.def_server, key="server", size=(25, 1)),
            ],
            [
                sg.Text("Disable SSL certificate check: ", size=(30, 1)),
                sg.Checkbox(
                    "(only check if using your own server)",
                    default=not Config.VERIFY_SSL_CERT,
                    key="ignore_cert",
                ),
            ],
            [
                sg.Text("Username: ", size=(30, 1)),
                sg.InputText(default_text=self.def_user, key="user", size=(25, 1)),
            ],
            [
                sg.Text("Password: ", size=(30, 1)),
                sg.InputText(default_text=self.def_pw, key="pw", size=(25, 1)),
            ],
            [sg.Text("")],
            [
                sg.Button("Login", size=(18, 1)),
                sg.Button("Register", size=(18, 1)),
                sg.Cancel(size=(18, 1)),
            ],
        ]
        return layout

    def show_clip_list_window(self, clips):
        """ Show all received clips in a Windows with a Listview and allow
            user select and copy an entry
        """
        # TODO: Show Only preview of Clip to reduce length
        layout = [
            [
                sg.Listbox(
                    values=clips,
                    size=(60, 6),
                    select_mode="LISTBOX_SELECT_MODE_SINGLE",
                    key="sel_clip",
                )
            ],
            [sg.Button("Copy to clipboard", size=(20, 1)), sg.Cancel(size=(20, 1))],
        ]
        window = sg.Window(
            title=f"{Config.APP_NAME} - Your Clips",
            layout=layout,
            size=(600, 300),
            icon=Config.ICON_B64,
        )
        event, values = window.read()
        if event in (sg.WIN_CLOSED, "Cancel"):
            log.debug("List Clips selection canceled")
            pass
        elif event == "Copy to clipboard":
            Api.paste(values.get("sel_clip")[0])
        window.close()

    def is_valid_server_address(self, server):
        """ Does server address match the format?
        """
        if not re.match(Config.MATCH_SERVER, server):
            return False
        return True

    def ask_for_cred(self):
        """ Create Windows asking for credentials
        """
        window = sg.Window(
            title=f"{Config.APP_NAME} - Enter credentials for registration / login",
            layout=self.create_cred_layout(),
            size=(500, 150),
            icon=Config.ICON_B64,
        )
        while not self.check_cred(window):
            pass
        log.debug("Ask for creds loop exited")
        window.close()
        return True

    def check_cred(self, window):
        """ Given credentials input check if we can perform action of
            login or registration on server

        Args:
            window (PySimpleGUIQt.Window): GUI-Window asking for creds

        Returns:
            bool: True if credentials are valid, else False
        """
        event, values = window.read()
        log.debug(f"{event} - {values}\n\n")
        if event in (sg.WIN_CLOSED, "Cancel"):
            log.debug("Credentials entry canceled")
            return True
        else:
            self.set_config_ignore_ssl_cert(values.get("ignore_cert"))
            if self.is_cred_input_valid(values):
                server, user, pw = self.is_cred_input_valid(values)
                if event == "Login":
                    if self.check_cred_login_and_save(server, user, pw):
                        sg.popup(
                            f"Credentials saved to config file: {Config.PATH_CONFIG_FILE}",
                            title=f"{Config.APP_NAME}",
                        )
                        return True
                elif event == "Register":
                    if self.check_cred_register_and_save(server, user, pw):
                        sg.popup(
                            f"Credentials saved to config file: {Config.PATH_CONFIG_FILE}",
                            title=f"{Config.APP_NAME}",
                        )
                    return True
        return False

    def set_config_ignore_ssl_cert(self, ignore_cert):
        """ Set config option to ignore ssl certificate validty in requests

        Args:
            ignore_cert (bool): Ignore SSL certificate
        """
        Config.VERIFY_SSL_CERT = not ignore_cert

    def is_cred_input_valid(self, values):
        """ Are credentials complete and server address valid?
        """
        server = values.get("server")
        if server == "":
            server = Config.DEFAULT_SERVER_URI
        user = values.get("user")
        pw = values.get("pw")
        if user == "" or pw == "":
            sg.popup(
                f"Please enter an username and password", title=f"{Config.APP_NAME}",
            )
            return False
        if not self.is_valid_server_address(server):
            sg.popup(
                f"Invalid server address: {server}\nUse format https://domain.tld:port",
                title=f"{Config.APP_NAME}",
            )
            return False
        if len(pw) < Config.MIN_PW_LENGTH:
            sg.popup(
                f"Your password must have at least 8 characters ",
                title=f"{Config.APP_NAME}",
            )
            return False
        log.debug(f"{server} - {user} - {pw}")
        return server, user, pw

    def check_cred_login_and_save(self, server, user, pw):
        """ Can we login using credentials? If so, save to configfile
        """
        try:
            Api.login(server, user, pw)
        except LoginException as e:
            log.error(f"Could not log in.\nPW: {pw}\nError: {e}")
            answer = sg.popup_yes_no(
                f"Login failed\n\nServer: {server}\nUser: {user}\nPassword: {pw}\n\n"
                f"Message: {e}\n\n"
                "Still save your credentials to the config file?",
                title=f"{Config.APP_NAME}",
            )
            if answer == "Yes":
                log.debug("Still saving creds")
                Config.write_config(server, user, pw)
                return True
            else:
                log.debug("Retry to login")
                return False
        else:
            log.debug("Logged in. Writing config")
            Config.write_config(server, user, pw)
            return True

    def check_cred_register_and_save(self, server, user, pw):
        """ Can we register using credentials? If so, save to configfile
        """
        try:
            Api.register(server, user, pw)
        except RegisterException as e:
            log.error(f"Could not register.\nPW: {pw}\nError: {e}")
            answer = sg.popup_yes_no(
                f"Registration failed\n\nServer: {server}\nUser: {user}\nPassword: {pw}\n\n"
                f"Message: {e}\n\n"
                "Still save your credentials to the config file?",
                title=f"{Config.APP_NAME}",
            )
            if answer == "Yes":
                log.debug("Still saving creds")
                Config.write_config(server, user, pw)
                return True
            else:
                log.debug("Retry to register")
                return False
        else:
            log.debug("Registration OK. Writing config")
            Config.write_config(server, user, pw)
            return True



import requests
import pyperclip
import json

try:
    # for package import
    from .log_config import log
    from .config import Config
    from .crypt import Crypt
except ModuleNotFoundError:
    # for direct call of clipster.py
    from log_config import log
    from config import Config
    from crypt import Crypt


class ApiException(Exception):
    """ All exceptions occuring when dealing with API
    """

    pass


class RegisterException(Exception):
    """ Could not register user
    """

    pass


class LoginException(Exception):
    """ Could not login user
    """

    pass


class Api:
    """ Deal with API requests and local clipboard
    """

    SERVER = None
    USER = None
    HASH_LOGIN = None
    HASH_MSG = None
    crypto = None

    def __init__(self, server, user, hash_login, hash_msg):
        self.SERVER = server
        self.USER = user
        self.HASH_LOGIN = hash_login
        self.HASH_MSG = hash_msg
        self.crypto = Crypt(
            username=user, password=None, hash_login=hash_login, hash_msg=hash_msg
        )

    def upload(self):
        """
        Send the copied text to SERVER
        """
        clip = self.copy()
        clip_encrypted = self.crypto.encrypt(clip)
        payload = {"text": clip_encrypted, "device": f"{Config.DEVICE_ID}"}
        try:
            res = requests.post(
                self.SERVER + Config.API_COPY_PASTE,
                data=payload,
                auth=(self.USER, self.HASH_LOGIN),
                timeout=Config.CONN_TIMEOUT,
                verify=Config.VERIFY_SSL_CERT,
                headers=Config.HEADERS,
            )
        except requests.exceptions.RequestException as e:
            log.exception("Error in upload request")
            raise ApiException(e)
        else:
            if res.status_code == 201:
                log.info("Success! Copied to Cloud-Clipboard.")
                return clip
            else:
                log.error(f"Error cannot upload clip: {res.text}")
                raise ApiException(res.text[0 : Config.MAX_RESPONSE_LEN])

    def download(self):
        """
        Download last or all clips from SERVER and updates the local clipboard
        """
        log.info("downloading clips")
        url = self.SERVER + Config.API_COPY_PASTE
        try:
            res = requests.get(
                url=url,
                auth=(self.USER, self.HASH_LOGIN),
                timeout=Config.CONN_TIMEOUT,
                verify=Config.VERIFY_SSL_CERT,
                headers=Config.HEADERS,
            )
        except requests.exceptions.RequestException as e:
            log.exception("Error in download request")
            raise ApiException(e)
        else:
            if res.status_code == 200:
                clips_decrypted = self.parse_and_decrypt_response(res)
                log.info(f"Got new clips from SERVER:\n{clips_decrypted}")
                self.paste(clips_decrypted[-1])
                return clips_decrypted
            else:
                log.error(f"Cannot download clips: {res.status_code} - {res.text}")
                raise ApiException(res.text[0 : Config.MAX_RESPONSE_LEN])

    def parse_and_decrypt_response(self, response):
        """ Parse list clip response and decrypt content
        Returns:
            List: Contains one or more clips. Ordered by creation date (DESC)
        """
        clips_decrypted = []
        try:
            clips = json.loads(response.text)
            for clip in clips:
                clips_decrypted.append(self.crypto.decrypt(clip["text"]))
            if len(clips) == 0:
                clips_decrypted = ["There are no shared Clips yet"]
        except Exception as e:
            log.e(f"Could not parse and decrypt: {e}")
            clips_decrypted = [""]
        return clips_decrypted

    @staticmethod
    def paste(data):
        """
        Copies 'data' to local clipboard which enables pasting.
        """
        pyperclip.copy(data)

    @staticmethod
    def copy():
        """
        Return the current clipboard text
        """
        data = pyperclip.paste()
        return data

    @staticmethod
    def register(server, user, pw):
        """
        register user on server using hash generated from pw
        """
        crypto = Crypt(user, pw)
        login_hash = crypto.pw_hash_login
        payload = {"username": user, "password": login_hash}
        try:
            res = requests.post(
                server + Config.API_REGISTER,
                data=payload,
                timeout=Config.CONN_TIMEOUT,
                verify=Config.VERIFY_SSL_CERT,
                headers=Config.HEADERS,
            )
        except requests.exceptions.RequestException as e:
            log.exception("Error in register request")
            raise RegisterException(e)
        if res.status_code == 201:
            log.info(f"Hi {user}! You are all set.")
            return True
        else:
            log.error(f"Cannot register user: {res.status_code} - {res.text}")
            raise RegisterException(res.text[0 : Config.MAX_RESPONSE_LEN])

    @staticmethod
    def login(server, user, pw):
        """
        authenticate user using hash generated from pw
        """
        crypto = Crypt(user, pw)
        login_hash = crypto.pw_hash_login
        try:
            res = requests.get(
                server + Config.API_LOGIN,
                auth=(user, login_hash),
                timeout=Config.CONN_TIMEOUT,
                verify=Config.VERIFY_SSL_CERT,
                headers=Config.HEADERS,
            )
        except requests.exceptions.RequestException as e:
            log.exception("Error in login request")
            raise LoginException(e)
        if res.status_code >= 200 and res.status_code < 400:
            log.info("Login successful")
            return True
        else:
            log.error(f"Login failed: {res.status_code} - {res.text}")
            raise LoginException(res.text[0 : Config.MAX_RESPONSE_LEN])



import logging
import logging.handlers
import platform
from pathlib import Path
import os

LEVEL = logging.DEBUG
current_os = platform.system()
# On Linux and Mac use /tmp
LOG_FILE = "/tmp/clipster.log"
if current_os == "Windows":
    LOG_FILE = Path(os.path.expandvars("%TEMP%")) / "clipster.log"

# Setup logging to rotating files of 10MB and console output
log = logging.getLogger("clipster")
log.setLevel(LEVEL)

ch = logging.StreamHandler()

rfh = logging.FileHandler(LOG_FILE)
rfh = logging.handlers.RotatingFileHandler(LOG_FILE, maxBytes=10097152, backupCount=5)

ch.setLevel(LEVEL)
rfh.setLevel(LEVEL)

formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")

ch.setFormatter(formatter)
rfh.setFormatter(formatter)
log.addHandler(ch)
log.addHandler(rfh)



import platform
import configparser
import PySimpleGUIQt as sg
from pathlib import Path

try:
    # for package import
    from .log_config import log
    from .crypt import Crypt
except ModuleNotFoundError:
    # for direct call of clipster.py
    from log_config import log
    from crypt import Crypt


class Config:

    APP_NAME = "Clipster"
    DEFAULT_SERVER_URI = "https://clipster.cc"
    SHOW_MESSAGE_DURATION = 2000
    CONN_TIMEOUT = 6
    MATCH_NONWHITESPACE = r"\S.*"
    MATCH_SERVER = (
        r"^(https):\/\/[^\s\/$.?#].[^\s]*|http://localhost:|http://127.0.0.1:"
    )
    DEVICE_ID = f"desktop_{platform.node()}"
    PATH_CONFIG_DIR = Path.home() / ".config/clipster/"
    PATH_CONFIG_FILE = PATH_CONFIG_DIR / "config"
    CONFIGFILE_MTIME = None
    MAX_NOTIFY_LEN = 60
    MAX_RESPONSE_LEN = 400
    MAX_CLIP_PREVIEW_LEN = 200
    MIN_PW_LENGTH = 8

    HEADERS = {"Accept": "application/json"}
    API_COPY_PASTE = "/copy-paste/"
    API_REGISTER = "/register/"
    API_LOGIN = "/verify-user/"
    SERVER = None
    USER = None
    PW = None
    PW_HASH_LOGIN = None
    PW_HASH_MSG = None
    VERIFY_SSL_CERT = True

    def __init__(self):
        pass

    @classmethod
    def was_configfile_modified(cls):
        """ Check if configfile has been changed since last modificatiom
        """
        log.debug("Checking if configfile has been modified")
        if cls.CONFIGFILE_MTIME:
            mtime = Path(cls.PATH_CONFIG_FILE).stat().st_mtime
            if mtime > cls.CONFIGFILE_MTIME:
                return True
            else:
                return False
        cls.CONFIGFILE_MTIME = Path(cls.PATH_CONFIG_FILE).stat().st_mtime
        return False

    @classmethod
    def is_configfile_valid(cls):
        """ Check if we have a valid config, read it and save values
        """
        log.debug("Validating config")
        conf = configparser.ConfigParser()
        if cls.PATH_CONFIG_FILE.exists():
            conf.read(cls.PATH_CONFIG_FILE)
            try:
                cls.SERVER = conf.get("settings", "server")
                cls.USER = conf.get("settings", "username")
                cls.PW_HASH_LOGIN = conf.get("settings", "hash_login")
                cls.PW_HASH_MSG = conf.get("settings", "hash_msg")
                cls.VERIFY_SSL_CERT = conf.getboolean("settings", "verify_ssl_cert")
            except (configparser.NoSectionError, KeyError):
                return False
            if cls.SERVER and cls.USER and cls.PW_HASH_LOGIN and cls.PW_HASH_MSG:
                log.debug("Config ok")
                return True
        return False

    @classmethod
    def write_config(cls, server, username, password):
        """ Write config file and save modification time
        """
        log.debug(
            f"Writing config file: {server} {username} {password} {cls.VERIFY_SSL_CERT}"
        )
        crypto = Crypt(username, password)
        cls.PATH_CONFIG_DIR.mkdir(parents=True, exist_ok=True)
        cls.PATH_CONFIG_FILE.touch(exist_ok=True)
        config = configparser.ConfigParser()
        config["settings"] = {
            "server": server,
            "username": username,
            "hash_login": crypto.pw_hash_login,
            "hash_msg": crypto.pw_hash_msg,
            "verify_ssl_cert": cls.VERIFY_SSL_CERT,
        }
        with open(cls.PATH_CONFIG_FILE, "w") as configfile:
            config.write(configfile)
        if not cls.CONFIGFILE_MTIME:
            cls.CONFIGFILE_MTIME = Path(cls.PATH_CONFIG_FILE).stat().st_mtime
        return True

    # PNG Icon 128x128 encoded with base64
    ICON_B64 = b"""iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAYAAADDPmHLAAAABmJLR0QA/wD/AP+gvaeTAAAACXBI
                WXMAAC4jAAAuIwF4pT92AAAAB3RJTUUH5AofFCUzT8cWAQAACRlJREFUeF7tnFtIFF8cx3+7rrne
                0jVXWZF0FQk1Y8XLahcI0x66EWQR9WI+SFbEBhFEGBRCEL1ZEEQ9RIGhRj0kBl2wmxe8RJqmheuW
                Ieuaa+3NVdff/yFanDMW5u4/Zz2/D3wfmjlzzpzffMzZM+PKEBGB4BY5u4HgCxKAc0gAziEBOIcE
                4BwSgHNIAM4hATiHBOAcEoBzSADOIQE4hwTgHBKAc0gAziEBOIcE4BwSgHNIAM4hATiHBOAcEoBz
                SADOIQE4hwTgHBKAc0gAziEBOIcE4BwSgHNIAM4hATiHBOAcEoBzSADOIQE4hwTgHBKAc0gAziEB
                OIcE4BwSgHNIAM4hATiHBOAcBbshUHC5XGC1WsFms0FfXx+Mjo7C8PAwOJ1OSEhIgKSkJEhPT4e4
                uDhQq9UQEhLCdvFHXC4XWCwWMJvNMDAwAEajESwWC0RFRUFiYiJoNBpIT0+HiIgIiImJ+ev+JQMG
                GF1dXXj8+HHMzc3FuLg4BIA/JiYmBvPz8/Hy5cs4MzPDdifCbrfj+fPnMScnB1evXi3qj41Go8GC
                ggI8ffo09vX1sd1JnoAQwOPxYH9/P+bk5IguwN8kMjISnz9/znbvpaGhAUNCQkTH/U2KiopweHgY
                PR4P270kkbwA3d3duHv3blGhfUl5ebngAjmdTty7d6+onS85ePAgDg0NzZuJNJG0AFeuXEGZTCYq
                rj+Sn5/vHWft2rWi/f6ITCbD27dvz5uR9JCkABMTE3jkyBFRQf2dXbt2YWFhoWi7v2MwGNDlcrHT
                lASSE+D79++YmZkpKmKgJz8/H51OJzvdZUdSApjNZkxJSREVb6WkoKAAJycn2WkvK5IRYGRkZEVf
                /F/JysrCb9++sdNfNiQhwNTUFGZkZIiKtVJTXFy8qDWJf8GyC2C3233+fB+IKSkpkcRawbIK4PF4
                8PDhw6Li8JJTp07h3NwcW5Z/yrIKcO7cOVFReEtNTQ1bln/KggLU19fjhQsX8NGjR+wuv1FTUyMq
                Bo+RyWRYV1fHlsdv1NXV4cWLF7GxsZHdhYjzBHA4HFhdXY1yuVxwggaDYX57v9DR0YGhoaGiYvCa
                NWvW4ODgIFsmn9m/f79gnIiICLx69argBtQrAHsXrlAosKSkBAEAe3t7vQf4ytDQEF38BaJWq9Fm
                s7HlWjKPHz9GgJ83m+xYW7du9bYDRMRjx44JGpSWlnob3Lt3D8+cOeP9ty+Mj49jWlqa6IQoP5Od
                nY12u50t25IoLS3FtrY2REScmZnBbdu2Cca6dOkSIiKCxWLBpKQk7w69Xj+/H0RErKqqYjctidzc
                XNGkKcLs2rWLLduSYH9oZ2dnBQ+91q9fjzabDaG/v19wAl1dXYIDEfGPz9AXw9TUFOr1etFkKQtn
                586dPn88XOia1dbWeseIiIhAo9GI8OzZM8Hg/mZ6ehq3b98umqQUkpycjAkJCaLtUsihQ4fYUvqM
                yWQSjPH+/XuEuro6wUar1coet2TMZjMmJiaKJieVfP36FTs6OkTbpZJ169ahw+Fgy7pkOjs7Bf33
                9PQgtLS0CDb+ujnwlYaGhkW9s7cciYqKEvyqa2pq8vlVsP8rWq0Wm5qa5lV26ZSXlwv67u/vRxgc
                HESFQuHdqFQq0Ww2s8cumpGRESwsLMSgoCDRZKSShV7eZH8VSinBwcFYUlLi06Pk3t5ewTVRqVT4
                +fNnhPHxcdRqtYIBExIS8MmTJzgxMcH2syBjY2P4+vVrya/rZ2VlocViYU/fi9FoFHwikmIqKiqw
                ra1t0Y+ULRYLPnz4EFUqlaCfDRs2oM1mQxki4r59++D+/fvAkpqaCjqdDrZs2QI6nQ5iY2Nh1apV
                YLPZwGQyQXd3N7x48QJMJhOYTCZARLYLyWAwGKC6uhrCw8PZXQImJyehsrISamtr2V2SQaFQQFJS
                EqSkpMCmTZsgJycHEhMTITw8HNxuN4yNjUFnZyc0NzdDb28vmEwmtgs4ceIE1NTUACAivnz5UmTa
                SolarcYHDx4IfioWw82bNzE6OlrU30rJx48fEXHeUjC7brwScvLkSfzy5Yv3ov4tg4ODWFZWJuo3
                0HP27FnvHL0C2Gw2jI+PFzUOxCQnJ+OHDx+8k/SV1tZWVKvVonECMZmZmYKXUwUrP01NTaIDAimF
                hYV4586d+VPyK9evX8fs7GzRuIESpVKJ7e3tgjmJlv4C8X4gOTkZm5ub0e12s9PxOy6XCxsbGzE2
                NlZ0HlJOcHDwgo+cRQIg/lxHZj8aSi0xMTG4Y8eO377o8C+ora3F4uLiRf0R6XImMzMTu7u72dNH
                xN8IgPjz0W1lZaWos+VORkYG3rp1C0dGRnx+YOIPZmdn0WQy4bVr1/63PzHzJVVVVX9cQPqtAL9o
                bm5GnU63bEulKpUK09LS0GAw4MDAAHt6kuPt27dYUVGBWq0Wo6KiRPP5FwkNDcWCggLs6elhT0+E
                DHFxqzcdHR1QV1cHN27cAKvVyu72G9HR0aDX62Hjxo2g1+tBq9VCamoqBAUFsU0lzfT0NHz69AmM
                RiO0tLTAmzdvoLW1FVwuF9vUb2g0Gjh69Cjs2bMHdDodu3tBFi3AfNrb2+Hu3btQX18PP378gNnZ
                WfB4PDA3NweICHNzc4L2crkcZDIZyGQyCAoKArlcDgqFAuRyOeTl5UFeXh5s3rwZioqKQKlUCo5d
                adhsNnj69Cm8evUK2tvb4d27d+DxeLz183g8gD//Z/5tHefXMDY2Fg4cOABlZWWQnp4uaL8YliTA
                fBwOB5hMJjCbzTAxMQFTU1MwMTEBbrcbAADCwsIgMjISwsLCQKlUQnx8PKhUKtBoNBAWFsb0xid2
                ux1GR0fBarWC2WwGt9sNDocD7HY7OJ1OkMvlEBIS4v0qGrVaDXFxcaDVan3+ahqfBSACG/qWMM4h
                ATiHBOAcEoBzSADOIQE4hwTgHBKAc0gAziEBOIcE4BwSgHNIAM4hATiHBOAcEoBzSADOIQE4hwTg
                HBKAc0gAziEBOIcE4BwSgHNIAM4hATiHBOAcEoBzSADOIQE4hwTgHBKAc0gAziEBOIcE4BwSgHNI
                AM4hATiHBOAcEoBzSADOIQE4hwTgHBKAc0gAzvkPY4FfWv2+aGoAAAAASUVORK5CYII=
                """



from setuptools import setup
import pathlib
import re

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")
# Remove gif - It won't be display correctly in PyPi
long_description = re.sub(r"!\[Clipster demo\].+?\)", "", long_description)

setup(
    name="clipster-desktop",
    version="0.4.5",
    description="Multi Platform Cloud Clipboard - Desktop Client",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="http://github.com/mc51/Clipster-Desktop",
    author="MC51",
    author_email="mc@data-dive.com",
    license="MIT",
    packages=["clipster"],
    install_requires=[
        "requests",
        "pyperclip",
        "PySide2",
        "PySimpleGUI",
        "PySimpleGUIQt",
        "cryptography",
    ],
    python_requires=">=3.6",
    zip_safe=False,
    scripts=["scripts/clipster_enable.sh", "scripts/clipster_disable.sh"],
    entry_points={"console_scripts": ["clipster = clipster.clipster:main"]},
    package_data={"clipster": ["resources/clipster.ico"]},
    classifiers=[
        "Topic :: Desktop Environment",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Utilities",
        "Topic :: System",
        "Topic :: Security :: Cryptography",
        "Development Status :: 4 - Beta",
        "Environment :: MacOS X",
        "Environment :: X11 Applications",
        "Environment :: Win32 (MS Windows)",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    keywords="clipster, clipboard, cryptography, secure, copy&paste, cloud, clipboard manager, data dive",
    project_urls={
        "Bug Reports": "https://github.com/mc51/Clipster-Desktop/issues",
        "Source": "https://github.com/mc51/Clipster-Desktop/",
        "Developer Website": "https://data-dive.com",
    },
)



import base64
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.fernet import InvalidToken

try:
    # for package import
    from .log_config import log
except ModuleNotFoundError:
    # for direct call of clipster.py
    from log_config import log


class EncryptException(Exception):
    """ All exceptions occuring when dealing with Crypt
    """


class DecryptException(Exception):
    """ All exceptions occuring when dealing with Crypt
    """


class Crypt:
    """ Symmetric encryption / decryption of clipboard data using Fernet
    """

    HASH_ITERS_LOGIN = 20000
    HASH_ITERS_MSG = 10000
    HASH_LENGTH = 32
    fernet = None
    pw_hash_login = None
    pw_hash_msg = None

    def __init__(self, username, password, hash_login=None, hash_msg=None):
        """ Create / Set hashes to be used for login and crypto by hashing the
            user's password. Initialize Fernet with crypto hash

        Args:
            username (str): Login cleartext username
            password (str): Login cleartext password
            hash_login (str): PW Hash for authentication
            hash_msg (str): PW Hash for crypto
        """
        self.pw_hash_login = hash_login
        self.pw_hash_msg = hash_msg
        if password:
            salt = f"clipster_{username}_{password}".encode()
            password = password.encode()
            self.pw_hash_login = self.get_hash(password, salt, self.HASH_ITERS_LOGIN)
            self.pw_hash_msg = self.get_hash(password, salt, self.HASH_ITERS_MSG)
        self.fernet = Fernet(self.pw_hash_msg)

    def get_hash(self, password, salt, iterations):
        """ Create PBKDF2 Hash of password

        Args:
            password (str): Password to be hashed
            salt (str): Salt used in hashing
            iterations (int): Hash Iterations

        Returns:
            str: Base64 urlsafe hash
        """
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=self.HASH_LENGTH,
            salt=salt,
            iterations=iterations,
        )
        hashh = kdf.derive(password)
        hashh = base64.urlsafe_b64encode(hashh)
        return hashh.decode()

    def encrypt(self, data):
        """ Returns encrypted text
        """
        data = data.encode()
        encrypted = self.fernet.encrypt(data)
        return encrypted

    def decrypt(self, data):
        """ Return encrypted text
        """
        data = data.encode()
        try:
            clear = self.fernet.decrypt(data)
        except InvalidToken as e:
            log.exception(f"ERROR Decrypt: {e}")
            clear = "Error: Could not decrypt received clip".encode()
        clear = clear.decode()
        return clear
